
#ifndef _itkComposeUniformTransformsTransform_txx
#define _itkComposeUniformTransformsTransform_txx

#include "itkComposeUniformTransformsTransform.h"


namespace itk
{

// Constructor with default arguments
template<class TScalarType,unsigned int NDimensions,class TContainerType>
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
ComposeUniformTransformsTransform():Superclass(NDimensions,0)
{
	m_Transforms = ContainerType::New();
}
    

// Destructor
template<class TScalarType,unsigned int NDimensions,class TContainerType>
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
~ComposeUniformTransformsTransform()
{
	return;
}


// Print self
template<class TScalarType,unsigned int NDimensions,class TContainerType>
void
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
PrintSelf(std::ostream &os, Indent indent) const
{
  Superclass::PrintSelf(os,indent);

	ContainerType::ConstIterator iterator = m_Transforms->Begin();
	ContainerType::ConstIterator end = m_Transforms->End();
	while (iterator != end) 
		{
			os << indent << "Transform \"" << iterator.Index() << "\":" << std::endl;
			iterator.Value()->Print(os,indent.GetNextIndent());
			++iterator;
		}
}


// Transform a point
template<class TScalarType,unsigned int NDimensions,class TContainerType>
typename ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::OutputPointType
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
TransformPoint(const InputPointType &point) const 
{
	OutputPointType opoint(point);

	ContainerType::ConstIterator iterator = m_Transforms->Begin();
	ContainerType::ConstIterator end = m_Transforms->End();
	while (iterator != end) 
		{
			opoint = iterator.Value()->TransformPoint(opoint);
			++iterator;
		}

		return opoint;
}

// Transform a vector
template<class TScalarType,unsigned int NDimensions,class TContainerType>
typename ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::OutputVectorType
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
TransformVector(const InputVectorType &vect) const 
{
	OutputVectorType ovect(vect);

	ContainerType::ConstIterator iterator = m_Transforms->Begin();
	ContainerType::ConstIterator end = m_Transforms->End();
	while (iterator != end) 
		{
			ovect = iterator.Value()->TransformVector(ovect);
			++iterator;
		}

  return  ovect;
}


// Transform a vnl_vector_fixed
template<class TScalarType,unsigned int NDimensions,class TContainerType>
typename ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::OutputVnlVectorType
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
TransformVector(const InputVnlVectorType &vect) const 
{
	OutputVnlVectorType ovect(vect);

	ContainerType::ConstIterator iterator = m_Transforms->Begin();
	ContainerType::ConstIterator end = m_Transforms->End();
	while (iterator != end) 
		{
			ovect = iterator.Value()->TransformVector(ovect);
			++iterator;
		}

  return  ovect;
}


// Transform a CovariantVector
template<class TScalarType,unsigned int NDimensions,class TContainerType>
typename ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::OutputCovariantVectorType
ComposeUniformTransformsTransform<TScalarType,NDimensions,TContainerType>::
TransformCovariantVector(const InputCovariantVectorType &vect) const 
{
	OutputCovariantVectorType ovect(vect);

	ContainerType::ConstIterator iterator = m_Transforms->Begin();
	ContainerType::ConstIterator end = m_Transforms->End();
	while (iterator != end) 
		{
			ovect = iterator.Value()->TransformCovariantVector(ovect);
			++iterator;
		}

  return  ovect;
}

} // namespace

#endif
