#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include <stdlib.h>

#include "itkImage.h"

#include "itkResampleImageFilter.h"
#include "itkLinearInterpolateImageFunction.h"
#include "itkComposeUniformTransformsTransform.h"
#include "itkRigid3DTransform.h"
#include "itkPermuteAxesTransform.h"
#include "itkPolarToCartesianTransform.h"


  const     unsigned int   Dimension = 3;
	typedef   float          RealPixelType;
  typedef itk::Image< RealPixelType,  Dimension >   RealImageType;


	typedef itk::ResampleImageFilter<RealImageType,RealImageType> ResampleFilterType;
	typedef itk::LinearInterpolateImageFunction<RealImageType,double> LinearInterpolatorType;
  typedef itk::VectorContainer< int, typename itk::Transform<double,Dimension,Dimension>::Pointer> TransformContainerType;
  typedef itk::ComposeUniformTransformsTransform<double,Dimension,TransformContainerType> ComposeTransformType;
	typedef itk::Rigid3DTransform<double> Rigid3DTransformType;
  typedef itk::PermuteAxesTransform<double,Dimension> PermuteAxesTransformType;
	typedef itk::PolarToCartesianTransform<double,Dimension> PolarToCartesianTransformType;




int main( int argc, char * argv[] )
{


	ResampleFilterType::Pointer multiResampler = ResampleFilterType::New();
  LinearInterpolatorType::Pointer linearInterpolator = LinearInterpolatorType::New();
	ComposeTransformType::Pointer composedTransforms = ComposeTransformType::New();
	PolarToCartesianTransformType::Pointer polarTransform = PolarToCartesianTransformType::New();
	PermuteAxesTransformType::Pointer permuteTransform = PermuteAxesTransformType::New();
	Rigid3DTransformType::Pointer rigidTransform = Rigid3DTransformType::New();
	TransformContainerType::Pointer transformContainer = TransformContainerType::New();

  
  double center[] = {0,0,0};
	rigidTransform->SetCenter(center);
	transformContainer->InsertElement(0,(TransformContainerType::Element)polarTransform);
	transformContainer->InsertElement(1,(TransformContainerType::Element)permuteTransform);
	transformContainer->InsertElement(2,(TransformContainerType::Element)rigidTransform);
	composedTransforms->SetTransforms(transformContainer);
	multiResampler->SetTransform(composedTransforms);
	multiResampler->SetInterpolator(linearInterpolator);
		

  PermuteAxesTransformType::PermuteOrderArrayType permutation;
	permutation[0] = 1; 
  permutation[1] = 2; 
  permutation[2] = 0;


  std::cout << "00 ********************************************"<<std::endl;

  polarTransform->Print(std::cout);
  permuteTransform->Print(std::cout);
  rigidTransform->Print(std::cout);

  transformContainer->Print(std::cout);

  composedTransforms->Print(std::cout);
  multiResampler->Print(std::cout);

  std::cout << "01 ********************************************"<<std::endl;

  multiResampler->SetTransform(rigidTransform);
  multiResampler->Print(std::cout);

  std::cout << "02 ********************************************"<<std::endl;

  double center2[] = {1,0,0};
  rigidTransform->SetCenter(center2);
  rigidTransform->Print(std::cout);
  multiResampler->Print(std::cout);

  std::cout << "03 ********************************************"<<std::endl;

  multiResampler->SetTransform(composedTransforms);
  multiResampler->Print(std::cout);

  std::cout << "04 ********************************************"<<std::endl;

  TransformContainerType::Pointer transformContainer2 = TransformContainerType::New();

  composedTransforms->SetTransforms(transformContainer2);
  transformContainer2->Print(std::cout);
  composedTransforms->Print(std::cout);
  multiResampler->Print(std::cout);

  std::cout << "05 ********************************************"<<std::endl;

  transformContainer2->InsertElement(0,(TransformContainerType::Element)permuteTransform);
  permuteTransform->Print(std::cout);
  transformContainer2->Print(std::cout);
  composedTransforms->Print(std::cout);
  multiResampler->Print(std::cout);

  std::cout << "06 ********************************************"<<std::endl;

  permuteTransform->SetOrder(permutation);
  permuteTransform->Print(std::cout);
  transformContainer2->Print(std::cout);
  composedTransforms->Print(std::cout);
  multiResampler->Print(std::cout);

  std::cout << "07 ********************************************"<<std::endl;



	return EXIT_SUCCESS;
}
